#! /bin/bash
RED='\033[0;31m' # Red
NC='\033[0m' # No Color
GREEN='\033[0;32m' # Green
YELLOW='\033[1;33m' # Yellow

function info() {
  echo -e "${GREEN}[INFO]:${NC} $*"
}

function warn() {
  echo -e "${YELLOW}[WARN]:${NC} $*" >&2
}

function error() {
  echo -e "${RED}[ERROR]:${NC} $*" >&2
  exit 1
}

SCRIPT_DIR=$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd)
#shellcheck disable=SC1090,SC1091
source "${SCRIPT_DIR}/modules/utils.sh"

export EXECUTE_ON_NODE="true"
export NAMESPACE="uipath-infra"
export RUNTIME_IMAGE="registry.uipath.com/sf-k8-utils-rhel:0.8"

export PATH=$PATH:/var/lib/rancher/rke2/bin:${SCRIPT_DIR}/bin:/usr/local/bin
if [[ -z "${KUBECONFIG}" ]];
then
  [[ -f "/etc/rancher/rke2/rke2.yaml" ]] && export KUBECONFIG="${KUBECONFIG:-/etc/rancher/rke2/rke2.yaml}"
  [[ -f "/var/lib/rancher/rke2/agent/kubelet.kubeconfig" ]] && export KUBECONFIG="${KUBECONFIG:-/var/lib/rancher/rke2/agent/kubelet.kubeconfig}"
fi

function display_usage() {
  echo "usage: $(basename "$0") [-hdekfsaq]"
  echo "  -d          Run in debug"
  echo "  -h          Display help"
  echo "  -e          Execute externally, Pass this flag if the script is not being run on any of the nodes in the cluster"
  echo "  -i          Executes only infra checks"
  echo "  -f          Executes only fabric checks"
  echo "  -s          Executes only services checks"
  echo "  -a          Executes all checks (this is default behavior)"
  echo "  -q          Prints only warnings and errors"
}

export EXECUTE_ALL_CHECKS=0
export EXECUTE_INFRA_CHECKS=0
export EXECUTE_FABRIC_CHECKS=0
export EXECUTE_SERVICES_CHECKS=0
export PRINT_ONLY_ERRORS=0

# Parse args utility
function parse_args() {
  while getopts 'hdeifsaq' flag "$@"; do
    case "${flag}" in
    d)
      info "Running in debug"
      set -x
      ;;
    h)
      display_usage
      exit 0
      ;;
    e)
      export EXECUTE_ON_NODE="false"
      ;;
    i)
      export EXECUTE_INFRA_CHECKS=1
      ;;
    f)
      export EXECUTE_FABRIC_CHECKS=1
      ;;
    s)
      export EXECUTE_SERVICES_CHECKS=1
      ;;
    a)
      export EXECUTE_ALL_CHECKS=1
      ;;
    q)
      export PRINT_ONLY_ERRORS=1
      ;;
    :)
      error "Invalid option: ${OPTARG} requires an argument."
      ;;
    *)
      error "Unexpected option ${flag}"
      ;;
    esac
  done
  shift $((OPTIND - 1))

  if [[ "${EXECUTE_FABRIC_CHECKS}|${EXECUTE_INFRA_CHECKS}|${EXECUTE_SERVICES_CHECKS}" == "0|0|0" ]];
  then
    export EXECUTE_ALL_CHECKS=1;
  fi

  export SCRIPT_ARGS=();
  [[ "${EXECUTE_INFRA_CHECKS}" == "1" ]] && SCRIPT_ARGS+=("-i")
  [[ "${EXECUTE_FABRIC_CHECKS}" == "1" ]] && SCRIPT_ARGS+=("-f")
  [[ "${EXECUTE_SERVICES_CHECKS}" == "1" ]] && SCRIPT_ARGS+=("-s")
  [[ "${EXECUTE_ALL_CHECKS}" == "1" ]] && SCRIPT_ARGS+=("-a")
  [[ "${PRINT_ONLY_ERRORS}" == "1" ]] && SCRIPT_ARGS+=("-q")
}

function create_service_account() {
  cat << EOF | kubectl apply -f - 1>/dev/null
apiVersion: v1
kind: ServiceAccount
metadata:
  name: diagnostics-tool
  namespace: "${NAMESPACE}"
---
apiVersion: rbac.authorization.k8s.io/v1
kind: ClusterRole
metadata:
  name: uipath-diagnostics-tool
rules:
- apiGroups:
  - ""
  resources:
  - persistentvolumeclaims
  verbs:
  - create
  - delete
  - get
- apiGroups:
  - metrics.k8s.io
  resources:
  - nodes
  verbs:
  - list
  - get
- apiGroups:
  - ""
  resources:
  - pods
  - pods/log
  - pods/exec
  verbs:
  - get
  - list
  - create
  - delete
- apiGroups:
  - apps
  resources:
  - deployments
  - statefulsets
  - daemonsets
  verbs:
  - get
  - list
- apiGroups:
  - batch
  resources:
  - jobs
  verbs:
  - get
  - list
- apiGroups:
  - ""
  resources:
  - nodes
  verbs:
  - get
  - list
- apiGroups:
  - networking.istio.io
  resources:
  - gateways
  - virtualservices
  verbs:
  - get
  - list
- apiGroups:
  - ""
  resources:
  - services
  - secrets
  verbs:
  - get
---
apiVersion: rbac.authorization.k8s.io/v1
kind: ClusterRoleBinding
metadata:
  name: uipath-diagnostics-tool
roleRef:
  apiGroup: rbac.authorization.k8s.io
  kind: ClusterRole
  name: uipath-diagnostics-tool
subjects:
- kind: ServiceAccount
  name: diagnostics-tool
  namespace: "${NAMESPACE}"
EOF
}

function delete_service_account() {
  kubectl -n ${NAMESPACE} delete ClusterRoleBinding uipath-diagnostics-tool &>/dev/null
  kubectl -n ${NAMESPACE} delete ClusterRole uipath-diagnostics-tool &>/dev/null
  kubectl -n ${NAMESPACE} delete serviceaccount diagnostics-tool &>/dev/null
}

function set_runtime_image_offline(){
  [[ -z "$(kubectl -n docker-registry get pod -l=app=docker-registry -o name)" ]] && return
  if [[ $(kubectl -n docker-registry get deploy docker-registry -o jsonpath="{.status.readyReplicas}") != "1" ]];
  then
    warn "Docker registry is not up"
    return 
  fi

  TAG=$(kubectl -n docker-registry exec -it "$(kubectl -n docker-registry get pod -l=app=docker-registry --field-selector=status.phase=Running -o name | head -1)" -- sh -c "ls /var/lib/registry/docker/registry/v2/repositories/sf-k8-utils-rhel/_manifests/tags 2>/dev/null | head -n 1")
  if [[ -n ${TAG} ]];
  then
    export RUNTIME_IMAGE="localhost:30071/sf-k8-utils-rhel:${TAG}"
  fi
}

# Creates the diagnostics job 
function create_diagnostics_job() {
  # Create service account required for diagnostics tool
  create_service_account

  delete_diagnostics_job
  set_runtime_image_offline

  kubectl -n ${NAMESPACE} create cm diagnostics-scripts --from-file="${SCRIPT_DIR}/job-scripts/" &>/dev/null
  kubectl -n ${NAMESPACE} create cm diagnostics-infra-components --from-file="${SCRIPT_DIR}/job-scripts/components/infra/" &>/dev/null
  kubectl -n ${NAMESPACE} create cm diagnostics-fabric-components --from-file="${SCRIPT_DIR}/job-scripts/components/fabric/" &>/dev/null
  kubectl -n ${NAMESPACE} create cm diagnostics-services-components --from-file="${SCRIPT_DIR}/job-scripts/components/services/" &>/dev/null
  kubectl -n ${NAMESPACE} create cm diagnostics-modules --from-file="${SCRIPT_DIR}/modules/" &>/dev/null

  cat << EOF | kubectl create -f - 1>/dev/null
apiVersion: batch/v1
kind: Job
metadata:
  name: diagnostics-tool
  namespace: "${NAMESPACE}"
spec:
  backoffLimit: 0
  template:
    metadata:
      labels:
        app.kubernetes.io/component: diagnostics-tool
      annotations:
        sidecar.istio.io/inject: "false"
    spec:
      serviceAccountName: diagnostics-tool
      containers:
      - name: diagnostics-tool
        image: ${RUNTIME_IMAGE}
        imagePullPolicy: Always
        command: ["bash", "/service-fabric/diagnostics-report.sh", "${SCRIPT_ARGS[*]}"]
        env:
          - name: BASE_PATH
            value: "/service-fabric"
        securityContext:
          privileged: false
          allowPrivilegeEscalation: false
          capabilities:
            drop: ["NET_RAW"]
        volumeMounts:
          - name: scripts
            mountPath: /service-fabric
          - name: infracomponents
            mountPath: /service-fabric/components/infra
          - name: fabriccomponents
            mountPath: /service-fabric/components/fabric
          - name: servicescomponents
            mountPath: /service-fabric/components/services
          - name: modules
            mountPath: /service-fabric/modules
          - name: tmpdir
            mountPath: /tmp
      restartPolicy: Never
      imagePullSecrets:
      - name: uipathpullsecret
      volumes:
      - name: scripts
        configMap:
          name: diagnostics-scripts
      - name: infracomponents
        configMap:
          name: diagnostics-infra-components
      - name: fabriccomponents
        configMap:
          name: diagnostics-fabric-components
      - name: servicescomponents
        configMap:
          name: diagnostics-services-components
      - name: modules
        configMap:
          name: diagnostics-modules
      - name: tmpdir
        emptyDir: {}
EOF
}

# Deletes the diagnostics job that was created
function delete_diagnostics_job() {
  kubectl -n "${NAMESPACE}" delete job diagnostics-tool &>/dev/null
  kubectl -n "${NAMESPACE}" delete cm diagnostics-scripts &>/dev/null
  kubectl -n "${NAMESPACE}" delete cm diagnostics-infra-components &>/dev/null
  kubectl -n "${NAMESPACE}" delete cm diagnostics-fabric-components &>/dev/null
  kubectl -n "${NAMESPACE}" delete cm diagnostics-services-components &>/dev/null
  kubectl -n "${NAMESPACE}" delete cm diagnostics-modules &>/dev/null
}

# Takes folder as argument and executes all .sh files from there
function execute_scripts(){
  for file in "$1"/*.sh ; do
    bash "${file}"
  done
}

function execute_diagnostics_checks() {
  if [[ -f ${REPORTS_LOCATION} ]];
  then
    rm "${REPORTS_LOCATION}"
  fi
  echo -ne "Running diagnostics checks......"
  add_to_report echo "---DIAGNOSTICS REPORT START---"

  [[ "${EXECUTE_INFRA_CHECKS}" == "1" || "${EXECUTE_ALL_CHECKS}" == "1" ]] && execute_scripts "${SCRIPT_DIR}/scripts/service-fabric/infra"
  [[ "${EXECUTE_FABRIC_CHECKS}" == "1" || "${EXECUTE_ALL_CHECKS}" == "1" ]] && execute_scripts "${SCRIPT_DIR}/scripts/service-fabric/fabric"
  [[ "${EXECUTE_SERVICES_CHECKS}" == "1" || "${EXECUTE_ALL_CHECKS}" == "1" ]] && execute_scripts "${SCRIPT_DIR}/scripts/service-fabric/services"
    
  add_to_report echo "---DIAGNOSTICS REPORT END---"
  sed -ne '/-DIAGNOSTICS REPORT START-/,/-DIAGNOSTICS REPORT END-/p' < /tmp/report.logs | sed '1d' | sed '$ d'
  echo ""
}

function execute_internal_diagnostics_checks() {
  create_diagnostics_job
  progress=".."

 retryCounter=0
 until [ "$retryCounter" -ge 30 ]
 do
     diagnosticsToolPod=$(kubectl -n ${NAMESPACE} get pods --selector job-name=diagnostics-tool --sort-by=.metadata.creationTimestamp | grep diagnostics-tool | tail -n -1)
     diagnosticsToolPodName=$(echo "$diagnosticsToolPod" | awk '{ print $1 }')
     diagnosticsToolPodStatus=$(kubectl -n ${NAMESPACE} get pod "$diagnosticsToolPodName" -o jsonpath="{.status.phase}")

     if [ "$diagnosticsToolPodStatus" = "Succeeded" ];
     then
         info "Internal diagnostics checks completed."
         kubectl -n ${NAMESPACE} logs "$diagnosticsToolPodName" | sed -ne '/-DIAGNOSTICS REPORT START-/,/-DIAGNOSTICS REPORT END-/p' | sed '1d' | sed '$ d'
         break
     fi

     if [ "$diagnosticsToolPodStatus" = "Failed" ];
     then
        warn "Failed to execute internal diagnostics checks"
        break
     fi

     echo -ne "Running internal diagnostics checks..${progress}\\r"
     progress="${progress}.."
     retryCounter=$((retryCounter+1)) 
     sleep 15
 done

 delete_diagnostics_job
 delete_service_account
}

function main() {

  parse_args "$@"

  if [[ "${EXECUTE_ON_NODE}" == "true" ]];
  then
    bash "${SCRIPT_DIR}/scripts/rke2/rke2-status.sh"
  fi

  if ! kubectl -n ${NAMESPACE} get pods &>/dev/null;
  then
    error "Kubectl command not working properly. Please check if current context is set correctly and cluster is responding."
    exit 1
  fi

  execute_diagnostics_checks

  execute_internal_diagnostics_checks
}

main "$@"