#! /bin/bash

RED='\033[0;31m' # Red
NC='\033[0m' # No Color
GREEN='\033[0;32m' # Green
YELLOW='\033[1;33m' # Yellow

export REPORTS_LOCATION=/tmp/report.logs

function info() {
  echo -e "${GREEN}[INFO]:${NC} $*"
}

function warn() {
  echo -e "${YELLOW}[WARN]:${NC} $*"
}

function error() {
  echo -e "${RED}[ERROR]:${NC} $*"
}

function section(){
    # Postpone printing section details, until something gets printed under this section
    SAVED_SECTION="$*"
}

function print_section(){
  printf "\n\n%.75s" "################### $* ################################################" >> ${REPORTS_LOCATION}
  printf "\n" >> ${REPORTS_LOCATION}
}

function execute(){
   eval "$@"
}

# Prints health of a property of node with colour coding
# 1 Property name of the node
# 2 Health status - False - healthy. True Not-Healthy
function print_health() {
    if [[ "${2}" = "False" ]];
    then
        add_to_report info "$1: False"
    else
        add_to_report error "$1: True"
    fi
}

# Use this function to send data to the report
# 1 - level of message : info, warn, error OR execute to execute the command and add output to report
# 2 - msg to be printed
function add_to_report(){
  COMMAND=$1
  MSG=$2

  [[ ${PRINT_ONLY_ERRORS} == 1 && "$COMMAND" == "info" ]] && return

  if [[ -n "${SAVED_SECTION}" ]];
  then
    print_section "${SAVED_SECTION}"
    SAVED_SECTION=""
  fi
  $COMMAND "$MSG" >> ${REPORTS_LOCATION}
}

# Checks the replicas of a deployment and prints it's status
# 1 - name of the deployment
# 2 - namespace
function check_replicas() {
    DESIRED_REPLICAS=$(kubectl -n "$2" get deploy "$1" -o json | jq .status.replicas)
    READY_REPLICAS=$(kubectl -n "$2" get deploy "$1" -o json | jq .status.readyReplicas)
    if [[ "${READY_REPLICAS}" = "${DESIRED_REPLICAS}" ]];
    then
        add_to_report info "Deployment $1 : Healthy"
    elif [[ "${READY_REPLICAS}" = "0" ]];
    then
        add_to_report error "Deployment $1 : Down."
    else
        if [[ "${READY_REPLICAS}" == "null" ]];
        then 
          READY_REPLICAS=0
        fi
        add_to_report warn "Deployment $1 : Number of ready replicas ${READY_REPLICAS} are less than desired ${DESIRED_REPLICAS}"
    fi
}

function contains() {
  if [[ $1 =~ (^|[[:space:]])$2($|[[:space:]]) ]]; then
    echo "true"
    return
  else
    echo "false"
    return
  fi
}

function get_cluster_config_key() {
  local key_name="$1"
  local value=""
  local config_map_name="service-cluster-configurations"
  local namespace="uipath-infra"

  configmap=$(kubectl get secret $config_map_name \
                --namespace $namespace \
                -o json \
                --ignore-not-found 2>> /dev/null)

  if [[ -n $configmap ]];
  then
    value=$(echo "${configmap}" | jq ".data.\"$key_name\"" | tr -d '"' | base64 -d)
  fi
  echo "${value}"
  return 0
}

function get_redis_cluster_name() {
  local namespace="${1:-redis-system}"
  local cluster_name=""
  cluster_name=$(kubectl -n "${namespace}" get rec \
                    -o jsonpath='{.items[0].metadata.name}' \
                    --ignore-not-found)
  echo "${cluster_name}"
  return 0
}

function capture_remote_cluster_info() {
  local remote_kubeconfig_path="${1}"
  local namespace="${2:-redis-system}"

  info "Reading the secret alternate-cluster-access from uipath-infra namespace"
  remote_config=$(kubectl get secret -n uipath-infra alternate-cluster-access -o jsonpath='{.data.alternate-cluster-kubeconfig}')
  [[ -z "${remote_config}" ]] && return 1
  echo "${remote_config}" | base64 -d > "${remote_kubeconfig_path}" || return 1

  REMOTE_SERVER_NAME=$(grep -E 'server:' "${remote_kubeconfig_path}"  | cut -d'/' -f 3 | cut -d':' -f1)
  [[ -z "${REMOTE_SERVER_NAME}" ]] && return 1

  REMOTE_REC=$(kubectl --request-timeout "20s" --kubeconfig="${remote_kubeconfig_path}" -n "${namespace}" get rec \
    -o jsonpath='{.items[0].metadata.name}' --ignore-not-found) || return 1
  info "Alternate redis cluster name: ${REMOTE_REC}"

  export REMOTE_REC
  export REMOTE_SERVER_NAME
  return 0
}