#! /bin/bash

SCRIPT_DIR=$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd)
#shellcheck disable=SC1090,SC1091
source "${SCRIPT_DIR}/../../../modules/utils.sh"

#Given a directory path, it will check if noexec option is set on the corresponding mount point
function noexec_flag_on_mountpoint() {
    local directory=$1
    mount_options=$(findmnt -T "${directory}" -o OPTIONS -n)
    if echo "$mount_options" | grep -q 'noexec';
    then
        add_to_report error "Validation failed: noexec flag is set on directory $directory, please remove it"
    else
        add_to_report info "Validation passed: noexec flag is not set on $directory"
    fi
 }

function validate_noexec_option() {
    section "Noexec flags on directories"
    noexec_flag_on_mountpoint "/var/lib/longhorn"
    noexec_flag_on_mountpoint "/var/lib/kubelet"
    noexec_flag_on_mountpoint "/var/lib/rancher"
}

# Prints the health details of the nodes
function node_health_status() {
    section "Nodes Health Status"
    nodes=$(kubectl get node --no-headers -o custom-columns=NAME:.metadata.name)
    for node in $nodes; do
        DISK_PRESSURE=$(kubectl get node "$node" -o json | jq '.status.conditions[] | select(."type"=="DiskPressure")' | jq .status | sed -e 's/^"//' -e 's/"$//')
        MEMORY_PRESSURE=$(kubectl get node "$node" -o json | jq '.status.conditions[] | select(."type"=="MemoryPressure")' | jq .status | sed -e 's/^"//' -e 's/"$//')
        print_health "$node: Disk Pressure " "$DISK_PRESSURE"
        print_health "$node: Memory Pressure " "$MEMORY_PRESSURE"

        ALLOCATED_CPU=$(kubectl describe node "$node" | grep -A 5 "Allocated resources" | grep cpu | awk -F "[()%]" '{print $2}')
        ALLOCATED_MEMORY=$(kubectl describe node "$node" | grep -A 5 "Allocated resources" | grep memory | awk -F "[()%]" '{print $2}')

        add_to_report info "$node: Allocated CPU percentage is $ALLOCATED_CPU%"
        add_to_report info "$node: Allocated memory percentage is $ALLOCATED_MEMORY%"

        NODE_UNSCHEDULABLE=$(kubectl describe node "$node" | grep Unschedulable | awk '{print $2}')
        if [[ "${NODE_UNSCHEDULABLE}" == "true" ]];
        then
            add_to_report error "$node is unschedulable"
        fi
    done

    section "Nodes CPU/RAM Usage"
    add_to_report execute "kubectl top nodes --use-protocol-buffers"
}

# Gets the total no of GPU enabled nodes in the cluster
function get_total_gpu_nodes() {
    section "GPU Nodes"
    nodes=$(kubectl get node --no-headers -o custom-columns=NAME:.metadata.name)
    TOTAL_GPU_NODES=0
    for node in $nodes; do
        GPU_CAPACITY=$(kubectl describe node "$node" | grep Capacity -A 7 | grep nvidia.com/gpu | cut -d: -f2 | sed 's/^[ \t]*//;s/[ \t]*$//')
        if [[ "${GPU_CAPACITY}" != "" ]];
        then
            TOTAL_GPU_NODES=$((TOTAL_GPU_NODES+1))
        fi
    done

    if [[ "${TOTAL_GPU_NODES}" -gt 0 ]];
    then 
        add_to_report info "Total number of GPU nodes in the cluster : ${TOTAL_GPU_NODES}"
    else
        add_to_report warn "There are no nodes with GPU in the cluster"
    fi
}

node_health_status
get_total_gpu_nodes
validate_noexec_option