#!/bin/bash
# Tools needed in container : s3cmd:2.1.0, bc, zip, gzip

DIR_LOCATION="/data/support-bundle"
BASE_PATH="/service-fabric"
RCLONE_CONFIG="/data/rclone.conf"
export RCLONE_CONFIG
DATE=$(date +'%Y_%m_%d')

info() {
  echo "[INFO] [$(date +'%Y-%m-%dT%H:%M:%S%z')]: $*"
}

warn() {
  echo "[WARN] [$(date +'%Y-%m-%dT%H:%M:%S%z')]: $*" >&2
}

error() {
  echo -e "\e[0;31m[ERROR][$(date +'%Y-%m-%dT%H:%M:%S%z')]:\e[0m $*" >&2
  exit 1
}
error_without_exit() {
  echo -e "\e[0;31m[ERROR][$(date +'%Y-%m-%dT%H:%M:%S%z')]:\e[0m $*" >&2
}

function display_usage() {
  echo "usage: $(basename "$0") [-hd] -i infile -o outfile -S S3Host -P S3Port -A S3AccessKey -K S3SecretKey -N namespaces -F startdate -D numberofdays -B bucketName"
  echo "  --debug                           Run in debug"
  echo "  --help                            Display help"
  echo "  --aws-host                        AWS_HOST value for S3 storage"
  echo "  --aws-port                        AWS_PORT value for S3 storage"
  echo "  --aws-accesskey                   AWS_ACCESS_KEY value for S3 storage"
  echo "  --aws-secretkey                   AWS_SECRET_KEY value for S3 storage"
  echo "  --aws-region                      AWS_REGION value for S3 storage"
  echo "  --aws-use-instance-profile        USE_INSTANCE_PROFILE value for S3 storage(true, false)"
  echo "  --azure-use-managed-identity      USE_MANAGED_IDENTITY value for azure storage(true, false)"
  echo "  --azure-accountname               Account name value for azure storage"
  echo "  --azure-accountkey                Account key value for azure storage"
  echo "  --azure-clientid                  client id value for azure storage if using managed identity"        
  echo "  --storage-type                    Storage type to be used(s3, ceph, azure)"
  echo "  --bucket                          Bucket name that holds the logs from the cluster"
  echo "  --namespaces                      Comma separated list of namespaces from which want to collect the data"
  echo "  --from-date                       Start date to collect the data, YYYY-MM-DD format. If start date is not mentioned, it will collect logs from last 2 days. Ex 2021-06-06"
  echo "  --days                            Number of days from the start date for which we want to collect the logs (default value is 7 days)"
}

# Parse args utility
function parse_args() {
  while (("$#")); do
    case "$1" in  
    --debug)
      info "Running in debug"
      shift
      set -x
      ;;
    --help)
      display_usage
      shift
      exit 0
      ;;
    --aws-host)
      if [ -n "$2" ] && [ "${2:0:1}" != "-" ]; then
        export AWS_HOST="$2"
        shift 2
      else
        display_usage
        error "Unexpected value for $1 parameter"
        shift 2
      fi
      ;;
    --aws-port)
      if [ -n "$2" ] && [ "${2:0:1}" != "-" ]; then
        export AWS_PORT="$2"
        shift 2
      else
        display_usage
        error "Unexpected value for $1 parameter"
        shift 2
      fi    
      ;;
    --aws-accesskey)
      if [ -n "$2" ] && [ "${2:0:1}" != "-" ]; then
        export AWS_ACCESS_KEY="$2"
        shift 2
      else
        display_usage
        error "Unexpected value for $1 parameter"
        shift 2
      fi    
      ;;
    --aws-secretkey)
      if [ -n "$2" ] && [ "${2:0:1}" != "-" ]; then
        export AWS_SECRET_KEY="$2"
        shift 2
      else
        display_usage
        error "Unexpected value for $1 parameter"
        shift 2
      fi    
      ;;
    --aws-region)
      if [ -n "$2" ] && [ "${2:0:1}" != "-" ]; then
        export AWS_REGION="$2"
        shift 2
      else
        display_usage
        error "Unexpected value for $1 parameter"
        shift 2
      fi    
      ;;
    --aws-use-instance-profile)
      if [ -n "$2" ] && [ "${2:0:1}" != "-" ]; then
        export AWS_USE_INSTANCE_PROFILE="$2"
        shift 2
      else
        display_usage
        error "Unexpected value for $1 parameter"
        shift 2
      fi    
      ;;
    --azure-use-managed-identity)
      if [ -n "$2" ] && [ "${2:0:1}" != "-" ]; then
        export AZURE_USE_MANAGED_IDENTITY="$2"
        shift 2
      else
        display_usage
        error "Unexpected value for $1 parameter"
        shift 2
      fi    
      ;;
    --azure-accountname)
      if [ -n "$2" ] && [ "${2:0:1}" != "-" ]; then
        export AZURE_ACCOUNTNAME="$2"
        shift 2
      else
        display_usage
        error "Unexpected value for $1 parameter"
        shift 2
      fi    
      ;;
    --azure-accountkey)
      if [ -n "$2" ] && [ "${2:0:1}" != "-" ]; then
        export AZURE_ACCOUNTKEY="$2"
        shift 2
      else
        display_usage
        error "Unexpected value for $1 parameter"
        shift 2
      fi    
      ;;
    --azure-clientid)
      if [ -n "$2" ] && [ "${2:0:1}" != "-" ]; then
        export AZURE_CLIENTID="$2"
        shift 2
      else
        display_usage
        error "Unexpected value for $1 parameter"
        shift 2
      fi    
      ;;
    --storage-type)
      if [ -n "$2" ] && [ "${2:0:1}" != "-" ]; then
        export STORAGE_TYPE="$2"
        shift 2
      else
        display_usage
        error "Unexpected value for $1 parameter"
        shift 2
      fi      
      ;;                              
    --namespaces)
      if [ -n "$2" ] && [ "${2:0:1}" != "-" ]; then
        export NAMESPACES="$2"
        shift 2
      else
        display_usage
        error "Unexpected value for $1 parameter"
        shift 2
      fi      
      ;;
    --bucket)
      if [ -n "$2" ] && [ "${2:0:1}" != "-" ]; then
        export BUCKET_NAME="$2"
        shift 2
      else
        display_usage
        error "Unexpected value for $1 parameter"
        shift 2
      fi      
      ;;
    --from-date)
      if [ -n "$2" ] && [ "${2:0:1}" != "-" ]; then
        export FROM_DATE="$2"
        shift 2
      else
        display_usage
        error "Unexpected value for $1 parameter"
        shift 2
      fi
      ;;
    --days)
      if [ -n "$2" ] && [ "${2:0:1}" != "-" ]; then
        export NUMBER_OF_DAYS="$2"
        shift 2
      else
        display_usage
        error "Unexpected value for $1 parameter"
        shift 2
      fi      
      ;;
    --only-upload)
        ONLY_UPLOAD="true"
        shift
      ;;
    --host-file-path)
      if [ -n "$2" ] && [ "${2:0:1}" != "-" ]; then
        export HOST_FILE_PATH="$2"
        shift 2
      else
        display_usage
        error "Unexpected value for $1 parameter"
        shift 2
      fi
      ;;
    :)
      error "Invalid option: ${1} requires an argument."
      ;;
    *)
      error "Unexpected option ${1}"
      ;;
    esac
  done

  if [ -z "$FROM_DATE" ];
  then
    FROM_DATE=$(date --date="1 day ago" +%Y-%m-%d)
    NUMBER_OF_DAYS=2
  fi

  if [ -z "${NUMBER_OF_DAYS}" ];
  then
    NUMBER_OF_DAYS=7
  fi

  if [[ "${ONLY_UPLOAD}" == "true" && -z "${HOST_FILE_PATH}" ]];
  then
    error "--host-file-path argument is mandatory when only-upload option is selected"
  fi
}

# Gets kubernetes objects yaml files 
# 1 Namespace
# 2 ObjectType - pod, deployment, daemonset etc
function getKubernetesObjects() {
  ns=$1
  objectType=$2
  info "Getting $objectType from $ns namespace"
  OBJECTS=$(kubectl -n "$ns" get "$objectType" --no-headers -o custom-columns=NAME:.metadata.name)
  for obj in $OBJECTS; do
    mkdir -p "$DIR_LOCATION/$objectType/$ns/"
    kubectl -n "$ns" get "$objectType" "$obj" -o yaml > "$DIR_LOCATION/$objectType/$ns/$obj.yaml"
  done
}

# get the download link for the uploaded bundle
function generate_signed_url_for_support_bundle() {
  local azure_client_id
  local file_path_in_bucket="$1"
  if [[ "${STORAGE_TYPE}" == "ceph" ]]; then
    SUPPORT_BUNDLE_URL=$(s3cmd signurl "s3://${BUCKET_NAME}/${file_path_in_bucket}" "$(echo "$(date +%s) + 3600 * 24 * 7" | bc)" --host="${AWS_HOST}" --host-bucket= --no-ssl)
  elif [[ "${STORAGE_TYPE}" == "s3" ]]; then
    SUPPORT_BUNDLE_URL=$(python3 "${BASE_PATH}/s3-signed-url.py" "${BUCKET_NAME}" "${file_path_in_bucket}" "${AWS_USE_INSTANCE_PROFILE}" "${AWS_ACCESS_KEY}" "${AWS_SECRET_KEY}" "https://${AWS_HOST}:${AWS_PORT}" "${AWS_REGION}" | grep SupportBundleLink | cut -d ' ' -f 2)
  elif [[ "${STORAGE_TYPE}" == "azure" ]]; then
    if [[ "${AZURE_USE_MANAGED_IDENTITY}" == "true" ]]; then
      [[ "${AZURE_CLIENTID}" == "CLIENT_ID" ]] && azure_client_id="" || azure_client_id="${AZURE_CLIENTID}"

      SUPPORT_BUNDLE_URL=$(python3 "${BASE_PATH}/azure-managed-signed-url.py" "${AZURE_ACCOUNTNAME}" "${BUCKET_NAME}" "${file_path_in_bucket}" "${azure_client_id}" ".blob.core.windows.net" | grep SupportBundleLink | cut -d ' ' -f 2)
    else
      SUPPORT_BUNDLE_URL=$(python3 "${BASE_PATH}/azure-non-managed-signed-url.py" "${AZURE_ACCOUNTNAME}" "${AZURE_ACCOUNTKEY}" "${BUCKET_NAME}" "${file_path_in_bucket}" ".blob.core.windows.net" | grep SupportBundleLink | cut -d ' ' -f 2)
    fi
  else
    error "Unknown storage type provided: ${STORAGE_TYPE}"
  fi
}

function uploadSupportBundleToDestination() {
  local file_path="$1"
  local destination_path="$2"
  rclone copy "${file_path}" remote-destination:"${destination_path}" --no-check-certificate --s3-upload-cutoff 0
}

# Uploads $SERVICE_BUNDLE_ZIP to the S3 storage
function uploadSupportBundle() {
    local file="$1"
    # upload the zip file to storage
    uploadSupportBundleToDestination "/data/$file" "${BUCKET_NAME}/support-bundles/$DATE/"
    generate_signed_url_for_support_bundle "support-bundles/${DATE}/$file"
}

# Consolidate all the logs podwise
function consolidateLogs() {
  LOGS_DIRECTORY="$DIR_LOCATION/logs"
  # Logs structure is /date/namespace/pod.container/logsfiles
  all_folders=$(ls .)
  for folder in $all_folders
  do
    if [ -d "$folder" ];
    then
      info "Processing logs for the date $folder"
      cd "$folder" || exit
      namespaces=$(ls .)
      for ns in $namespaces
      do
        if [[ "$NAMESPACES" != *"$ns"* ]];
        then
          # If namespace is not in the list provided, skip it
          continue
        fi
        info "Processing $ns namespace logs"
        if [ -d "$ns" ];
        then
          cd "$ns" || exit
          pods=$(ls .)
          for pod in $pods
          do
            if [ -d "$pod" ];
            then
              cd "$pod" || exit
              pod_name=$(echo "$pod" | cut -d '.' -f 1)
              container_name=$(echo "$pod" | cut -d '.' -f 2)
              mkdir -p "$LOGS_DIRECTORY/$ns/$pod_name"
              LOG_LOCATION=$LOGS_DIRECTORY/$ns/$pod_name/$container_name.log
              all_chunks=$(ls .)
              # For each container, append logs to the file at location /support-bundle/logs/namespace/pod/container.logs
              for chunk in $all_chunks
              do
                  gzip -d "$chunk"
                  file_name=$(echo "$chunk" | cut -d '.' -f 1)
                  cat "$file_name" >> "$LOG_LOCATION"
              done
              cd ..
            fi
          done
          cd ..
        fi
      done
      cd ..
    fi
  done
}


# Collect firing alerts from Prometheus
function collectClusterAlerts() {
  mkdir -p ${DIR_LOCATION}/alerts/
  info "Getting active alerts from Prometheus"
  cluster_ip=$(kubectl -n cattle-monitoring-system get services/rancher-monitoring-prometheus -o jsonpath='{.spec.clusterIP}') || error_without_exit "Failed to get cluster_ip for rancher-monitoring-prometheus"
  active_alerts=$(curl "http://${cluster_ip}:9090/api/v1/alerts") || error_without_exit "Curl failed while fetching alerts"
  filtered_alerts="$(jq -r '.data.alerts | map(select((.state == "firing") and (.labels.severity == "warning")))' <<< "$active_alerts")" 
  echo "${filtered_alerts}" > ${DIR_LOCATION}/alerts/warning.json
  filtered_alerts="$(jq -r '.data.alerts | map(select((.state == "firing") and (.labels.severity == "critical")))' <<< "$active_alerts")" 
  echo "${filtered_alerts}" > ${DIR_LOCATION}/alerts/critical.json
}

# Collect rest of the kuberntes object details, status of deployments, jobs, etc
function collectClusterDetails() {
  collectClusterAlerts || error_without_exit "Failed to fetch active alerts"
  # Get current logs from the pods and different objects yaml files to see the status and any events
  echo "$NAMESPACES" | tr "," "\n" | while read -r ns
  do
    info "Processing objects in $ns namespace"

    # Get logs from the current states of the pods for the last 4 hours
    PODS=$(kubectl -n "$ns" get pods --no-headers -o custom-columns=NAME:.metadata.name)
    info "Getting current logs from $ns namespace"
    for pod in $PODS; do
      CONTAINERS=$(kubectl -n "$ns" get pod "$pod" -o jsonpath="{.spec.containers[*].name}")
      for container in $CONTAINERS; do
        mkdir -p "$DIR_LOCATION/current-logs/$ns/$pod/"
        kubectl -n "$ns" logs "$pod" "$container" --since=4h >> "$DIR_LOCATION/current-logs/$ns/$pod/$container.log" || echo "The container $pod/$container logs could not be collected"
      done
      mkdir -p "$DIR_LOCATION/pods/$ns/"
      kubectl -n "$ns" get pod "$pod" -o yaml > "$DIR_LOCATION/pods/$ns/$pod.yaml"
    done
    
    # Get Kubernets yaml files of different Objects. Deployments, jobs, daemonsets
    getKubernetesObjects "$ns" "deployments"

    getKubernetesObjects "$ns" "jobs"

    getKubernetesObjects "$ns" "daemonsets"

    getKubernetesObjects "$ns" "services"

    # Get events from the namespaces
    mkdir -p $DIR_LOCATION/events/
    kubectl -n "$ns" get events -o json | jq 'del(.items[].metadata.managedFields)' > "$DIR_LOCATION/events/$ns.json"
  done
  # get the nodes details
  kubectl describe nodes > $DIR_LOCATION/nodes.info

  echo "Support bundle generated from ceph store" > ${DIR_LOCATION}/rook-ceph-up.txt

  # collect Identity Service support bundle 
  uipathNs="uipath"
  mkdir -p "$DIR_LOCATION/configmaps/$uipathNs"
  for identity_cm in "identity-service" "identity-service-clients" "identity-service-customconfig"
  do 
    kubectl -n "$uipathNs" get cm "$identity_cm" -o yaml > "$DIR_LOCATION/configmaps/$uipathNs/$identity_cm.yaml"
  done
}

# Get the Redis Enterprise support bundle
function getRedisSupportBundle(){
  local bundle_folder="${DIR_LOCATION}/redis_enterprise"
  local namespace="redis-system"
  local container="redis-enterprise-node"
  local log_file
  local pods

  info "Generating the Redis Enterprise support bundle"

  mkdir -p "${bundle_folder}" || true
  pods="$(kubectl get pods -n ${namespace} -l "redis.io/role=node" --no-headers -o custom-columns=NAME:.metadata.name)"
  for pod in ${pods}
  do 
    info "Getting the redis support bundle from pod: ${pod}"
    kubectl exec -n "${namespace}" "${pod}" -c "${container}" -- /opt/redislabs/bin/debuginfo &>/dev/null || true
    log_file="$(kubectl exec -n "${namespace}" "${pod}" -c "${container}" -- ls -t /tmp | grep debuginfo | head -1)"
    kubectl cp -c "${container}" ${namespace}/"${pod}":/tmp/"${log_file}" "${bundle_folder}/${pod}.tar.gz" || true
  done
}

function configure_destination_rclone() {
  if [[ "${STORAGE_TYPE}" == "ceph" ]]; then
    configure_ceph_rclone
  elif [[ "${STORAGE_TYPE}" == "s3" ]]; then
    configure_s3_rclone
  elif [[ "${STORAGE_TYPE}" == "azure" ]]; then
    configure_azure_rclone
  else
    error "Unknown storage type provided: ${STORAGE_TYPE}"
  fi
}

function configure_ceph_rclone() {
  rclone config create remote-destination s3 provider=Other \
  access_key_id="${AWS_ACCESS_KEY}" secret_access_key="${AWS_SECRET_KEY}" \
  endpoint="http://${AWS_HOST}:${AWS_PORT}" region="us-east-1" acl="private"
}

function configure_s3_rclone() {
  if [[ "${AWS_USE_INSTANCE_PROFILE}" == "true" ]]; then
    rclone config create remote-destination s3 provider=AWS \
    location_constraint="${AWS_REGION}" region="${AWS_REGION}" \
    acl="private" env-auth="true"
  else
    url="https://${AWS_HOST}"
    [[ "${AWS_PORT}" != "443" ]] && url="${url}:${AWS_PORT}"
    rclone config create remote-destination s3 provider=Other \
    access_key_id="${AWS_ACCESS_KEY}" secret_access_key="${AWS_SECRET_KEY}" \
    endpoint="${url}" location_constraint="${AWS_REGION}" region="${AWS_REGION}" acl="private"
  fi
}

function configure_azure_rclone() {
  if [[ "${AZURE_USE_MANAGED_IDENTITY}" == "true" ]]; then
    azure_msi_client_id_field=""
    [[ "${AZURE_CLIENTID}" == "CLIENT_ID" ]] && AZURE_CLIENTID=""
    [[ -n "${AZURE_CLIENTID}" ]] && azure_msi_client_id_field="msi_client_id  ${AZURE_CLIENTID}"

    #shellcheck disable=SC2086
    rclone config create remote-destination azureblob \
    account="${AZURE_ACCOUNTNAME}" use_msi "true" ${azure_msi_client_id_field} > /dev/null
  else
    rclone config create remote-destination azureblob \
    account="${AZURE_ACCOUNTNAME}" key="${AZURE_ACCOUNTKEY}" > /dev/null
  fi
}

function fail_if_url_is_not_valid() {
  local url="$1"
  if curl -k -l --output /dev/null --fail "${url}"; then
    echo "SupportBundleLink $url"
  else
    exit 1
  fi
}

function main() {
  parse_args "$@"

  configure_destination_rclone
  SERVICE_BUNDLE_ZIP="service-fabric-logs-$(date +'%Y-%m-%d-%H-%M-%S').zip"
  info "Bucket name is $BUCKET_NAME"
  if [[ "${ONLY_UPLOAD}" == "true" ]]; then
    mv "${HOST_FILE_PATH}" "/data/$SERVICE_BUNDLE_ZIP"
    uploadSupportBundleToDestination "/data/$SERVICE_BUNDLE_ZIP" "${BUCKET_NAME}/support-bundles/$DATE/"
    generate_signed_url_for_support_bundle "support-bundles/${DATE}/$SERVICE_BUNDLE_ZIP"
    fail_if_url_is_not_valid "$SUPPORT_BUNDLE_URL"
    return
  fi

  # Directory where all logs will be collected
  mkdir -p $DIR_LOCATION/logs

  # Temp directory where logs will be downloaded to
  mkdir -p /data/service-fabric-logs

  cd /data/service-fabric-logs || exit

  # Download logs for specified dates. From start date till number of days
  LOGS_DATE=$(date -d "$FROM_DATE")
  counter=0
  while [[ $counter -lt "$NUMBER_OF_DAYS" ]];
  do
    day=$(date -d "$LOGS_DATE + $counter days" +%Y_%m_%d)
    echo "$NAMESPACES" | tr "," "\n" | while read -r ns
    do
      mkdir -p "$day/$ns/"
      [[ "${STORAGE_TYPE}" == "ceph" ]] && source_folder="$BUCKET_NAME/$day/$ns/" || source_folder="$BUCKET_NAME/automation-suite-logs/$day/$ns/"
      rclone copy remote-destination:"${source_folder}" "$day/$ns/" --no-check-certificate
    done
    counter=$((counter+1)) 
  done

  # Consolidate all the logs at one place
  consolidateLogs

  # Collect rest of the kubernetes object details
  collectClusterDetails

  # Get the Redis Enterprise support bundle
  getRedisSupportBundle

  # Create a zip file out of the logs folder
  
  cd /data || return
  zip "/data/$SERVICE_BUNDLE_ZIP" support-bundle/* -r
  cd ..

  # Upload support bundle
  uploadSupportBundle "${SERVICE_BUNDLE_ZIP}"
  fail_if_url_is_not_valid "${SUPPORT_BUNDLE_URL}"
}

main "$@"
