#!/usr/bin/env bash

set -Eeou pipefail

#
# mdb_operator_diagnostic_data.sh
#
# Use this script to gather data about your MongoDB Community Kubernetes Operator
# and the MongoDB Resources deployed with it.
#

#
# shellcheck disable=SC2119
# shellcheck disable=SC2039
#

usage() {
    local script_name
    script_name=$(basename "${0}")
    echo "------------------------------------------------------------------------------"
    echo "Usage:"
    echo "${script_name} <namespace> <mongodb-replica-set> [--private]"
    echo "------------------------------------------------------------------------------"
    echo "#Scenario: Collecting MongoDB Logs:"
    echo "------------------------------------------------------------------------------"
    echo "Example: Deployment_Namespace: mongodb, Deployment_Name: mongodb-replica-set"
    echo "Usage: ${script_name} mongodb mongodb-replica-set"
}

if [ $# -lt 2 ]; then
    usage >&2
    exit 1
fi

namespace="${1}"
mdb_resource="${2}"
mdb_container_name="mongod"

operator_name="${4:-mongodb-kubernetes-operator}"
operator_namespace="${3:-$1}"

current_date="$(date +%Y-%m-%d_%H_%M)"

private_mode=1
contains "--private" "$@" && private_mode=0

log_dir="logs_${current_date}"
mkdir -p "${log_dir}" &> /dev/null

if ! kubectl get "namespace/${namespace}" &> /dev/null; then
    echo "Error fetching namespace. Make sure name ${namespace} for Namespace is correct."
    exit 1
fi

if ! kubectl -n "${namespace}" get "mongodbcommunity/${mdb_resource}" &> /dev/null; then
    echo "Error fetching the MongoDB resource. Make sure the ${mdb_resource} is correct."
    exit 1
fi

if ! kubectl -n "${operator_namespace}" get "deployment/${operator_name}" &> /dev/null; then
    echo "Error fetching the MongoDB Operator Deployment. Make sure the deployment/${operator_name} exist and it is running."
    exit 1
fi

if [ ${private_mode} == 0 ]; then
    echo "+ Running on private mode. Make sure you don't share the results of this run outside your organization."
fi

echo "++ Versions"
mdb_operator_pod=$(kubectl -n "${operator_namespace}" get pods -l "name=${operator_name}" -o name | cut -d'/' -f 2)
echo "+ Operator Pod: pod/${mdb_operator_pod}"

mdb_operator_filename="operator.yaml"
echo "+ Saving Operator Deployment into ${mdb_operator_filename}"
kubectl -n "${operator_namespace}" get "deployment/${operator_name}" -o yaml > "${log_dir}/${mdb_operator_filename}"

echo "+ Kubernetes Version Reported by kubectl"
kubectl version

if type oc &> /dev/null; then
    echo "+ Kubernetes Version Reported by oc"
    oc version
fi

operator_logs_filename="${operator_name}_${current_date}.logs"
echo "+ Saving Operator logs to file ${operator_logs_filename}"
kubectl -n "${operator_namespace}" logs "deployment/${operator_name}" --tail 2000 > "${log_dir}/${operator_logs_filename}"

operator_container_pretty_name=$(kubectl -n "${operator_namespace}" exec -it "${mdb_operator_pod}" -- sh -c "cat /etc/*release" | grep "PRETTY_NAME" | cut -d'=' -f 2)
echo "+ Operator is running on: ${operator_container_pretty_name}"

database_container_pretty_name=$(kubectl -n "${namespace}" exec -it "${mdb_resource}-0" -- sh -c "cat /etc/*release" | grep "PRETTY_NAME" | cut -d'=' -f 2)
echo "+ Database is running on: ${database_container_pretty_name}"

echo "++ Kubernetes Cluster Ecosystem"
echo "+ Kubectl Cluster Information"
kubectl cluster-info

if [ ${private_mode} == 0 ]; then
    kubectl_cluster_info_filename="kubectl_cluster_info_${current_date}.logs"
    echo "+ Saving Cluster Info to file ${kubectl_cluster_info_filename} (this might take a few minutes)"
    kubectl cluster-info dump | gzip > "${log_dir}/${kubectl_cluster_info_filename}.gz"
else
    echo "= Skipping Kubectl cluster information dump, use --private to enable."
fi

kubectl_sc_dump_filename="kubectl_storage_class_${current_date}.yaml"
kubectl get storageclass -o yaml > "${log_dir}/${kubectl_sc_dump_filename}"

nodes_filename="nodes.yaml"
echo "+ Nodes"
kubectl get nodes

echo "+ Saving Nodes full state to ${nodes_filename}"
kubectl get nodes -o yaml > "${log_dir}/${nodes_filename}"

echo "++ MongoDB Resource Running Environment"
crd_filename="crd_mdb.yaml"
echo "+ Saving MDB Customer Resource Definition into ${crd_filename}"
kubectl -n "${namespace}" get crd/mongodbcommunity.mongodbcommunity.mongodb.com -o yaml > "${log_dir}/${crd_filename}"

project_filename="project.yaml"
mdb_resource_name="mongodbcommunity/${mdb_resource}"
project_name=$(kubectl -n "${namespace}" get "${mdb_resource_name}" -o jsonpath='{.spec.security.tls.caConfigMapRef.name}')

resource_filename="mdb_object_${mdb_resource}.yaml"
echo "+ MongoDB Resource Status"
kubectl -n  "${namespace}" get "${mdb_resource_name}" -o yaml > "${log_dir}/${resource_filename}"

echo "+ Saving Project YAML file to ${project_filename}"
kubectl -n "${namespace}" get "configmap/${project_name}" -o yaml > "${log_dir}/${project_filename}"

credentials_user=$(kubectl -n "${namespace}" get "${mdb_resource_name}" -o jsonpath='{.spec.users[?(@.db=="admin")].name}')
echo "+ User configured is (credentials.user): ${credentials_user}"

statefulset_filename="statefulset.yaml"
echo "+ Saving StatefulSet state to ${statefulset_filename}"
kubectl -n "${namespace}" get "sts/${mdb_resource}" -o yaml > "${log_dir}/${statefulset_filename}"

echo "+ Deployment Pods"
kubectl -n "${namespace}" get pods | grep -E "^${mdb_resource}-[0-9]+"

echo "+ Saving Pods Events to ${mdb_resource}-N_events.logs"
pods_in_namespace=$(kubectl -n "${namespace}" get pods -o name | cut -d'/' -f 2 | grep -E "^${mdb_resource}-[0-9]+")
for pod in ${pods_in_namespace}; do
    kubectl -n "${namespace}" get event --field-selector "involvedObject.name=${pod}" > "${log_dir}/${pod}_events.log"
done

echo "+ Saving AppDB Pods state to ${mdb_resource}-N-<container_name>.logs and diagnostic data to ${mdb_resource}-N-diagnostic.data"
pods_in_namespace=$(kubectl -n "${namespace}" get pods -o name -l "app=${mdb_resource}-svc" | cut -d'/' -f 2)
for pod in ${pods_in_namespace}; do
    kubectl -n "${namespace}" logs "${pod}" -c "mongod" --tail 2000 > "${log_dir}/${pod}-mongod.log"
    kubectl -n "${namespace}" logs "${pod}" -c "mongodb-agent" --tail 2000 > "${log_dir}/${pod}-mongodb-agent.log"
    kubectl cp "${namespace}/$pod:data/diagnostic.data" -c ${mdb_container_name} "${log_dir}/${pod}-diagnostic.data/"
done

echo "+ Persistent Volumes"
kubectl -n "${namespace}" get pv

echo "+ Persistent Volume Claims"
kubectl -n "${namespace}" get pvc

pv_filename="persistent_volumes.yaml"
echo "+ Saving Persistent Volumes state to ${pv_filename}"
kubectl -n "${namespace}" get pv -o yaml > "${log_dir}/${pv_filename}"

pvc_filename="persistent_volume_claims.yaml"
echo "+ Saving Persistent Volumes Claims state to ${pvc_filename}"
kubectl -n "${namespace}" get pvc -o yaml > "${log_dir}/${pvc_filename}"

services_filename="services.yaml"
echo "+ Services"
kubectl -n "${namespace}" get services

echo "+ Saving Services state to ${services_filename}"
kubectl -n "${namespace}" get services -o yaml > "${log_dir}/${services_filename}"

echo "+ Saving Events for the Namespace"
kubectl -n "${namespace}" get events > "${log_dir}/events.log"

echo "+ Certificates (no private keys are captured)"
csr_filename="csr.text"
kubectl -n "${namespace}" get "certificates.cert-manager.io" || true
echo "+ Saving Certificate state into ${csr_filename}"
kubectl -n "${namespace}" get "certificates.cert-manager.io" -o name > "${log_dir}/${csr_filename}" || true

echo "++ MongoDBUser Resource Status"
mdbusers_filename="mdbu.yaml"
kubectl -n "${namespace}" get "${mdb_resource_name}" -o jsonpath='{.spec.users}'
echo "+ Saving MongoDBUsers to ${mdbusers_filename}"
kubectl -n "${namespace}" get "${mdb_resource_name}" -o jsonpath='{.spec.users}' > "${log_dir}/${mdbusers_filename}"

echo "- All logs have been captured"
echo "- If support is needed, please attach this file to an email to provide you with a better support experience."
echo "- If there are additional logs that your organization is capturing, they should be made available in case of a support request."

